import logging

from pprint import pformat

LOGGER = logging.getLogger(__name__)


class APIItems:
    """Base class for a map of API Items."""

    def __init__(self, raw, request, path, item_cls) -> None:
        self._request = request
        self._path = path
        self._item_cls = item_cls
        self._items = {}
        self.process_raw(raw)
        LOGGER.debug(pformat(raw))

    def update(self, path=None) -> None:
        path = path or self._path
        raw = self._request('get', path)
        self.process_raw(raw)

    def process_raw(self, raw: dict) -> None:
        for id, raw_item in raw.items():
            obj = self._items.get(id)

            if obj is not None:
                obj.raw = raw_item
            else:
                self._items[id] = self._item_cls(id, raw_item, self._request)

    def values(self):
        return self._items.values()

    def __getitem__(self, obj_id: str):
        return self._items[obj_id]

    def __iter__(self):
        return iter(self._items)



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

from .deconzdevice import DeconzDevice, DeconzDeviceSetter

_LOGGER = logging.getLogger(__name__)

ALARM = ['ZHAAlarm']
CARBONMONOXIDE = ['ZHACarbonMonoxide']
CONSUMPTION = ['ZHAConsumption']
DAYLIGHT = ['Daylight']
FIRE = ['ZHAFire']
GENERICFLAG = ['CLIPGenericFlag']
GENERICSTATUS = ['CLIPGenericStatus']
HUMIDITY = ['ZHAHumidity', 'CLIPHumidity']
LIGHTLEVEL = ['ZHALightLevel', 'CLIPLightLevel']
OPENCLOSE = ['ZHAOpenClose', 'CLIPOpenClose']
POWER = ['ZHAPower']
PRESENCE = ['ZHAPresence', 'CLIPPresence']
PRESSURE = ['ZHAPressure', 'CLIPPressure']
SWITCH = ['ZHASwitch', 'ZGPSwitch', 'CLIPSwitch']
TEMPERATURE = ['ZHATemperature', 'CLIPTemperature']
THERMOSTAT = ['ZHAThermostat', 'CLIPThermostat']
VIBRATION = ['ZHAVibration']
WATER = ['ZHAWater']

DECONZ_BINARY_SENSOR = CARBONMONOXIDE + FIRE + GENERICFLAG + OPENCLOSE + \
                       PRESENCE + VIBRATION + WATER
DECONZ_SENSOR = CONSUMPTION + DAYLIGHT + GENERICSTATUS + HUMIDITY + \
                LIGHTLEVEL + POWER + PRESSURE + SWITCH + TEMPERATURE
OTHER_SENSOR = THERMOSTAT


class DeconzSensor(DeconzDevice):
    """deCONZ sensor representation.

    Dresden Elektroniks documentation of sensors in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/sensors/
    """

    DECONZ_TYPE = '/sensors/'

    def __init__(self, device_id, device):
        """Set initial information about sensor."""
        self._battery = device['config'].get('battery')
        self._ep = device.get('ep')
        self._lowbattery = device['state'].get('lowbattery')
        self._on = device['config'].get('on')
        self._reachable = device['config'].get('reachable')
        self._tampered = device['state'].get('tampered')
        # self._temperature = device['config'].get('temperature')
        self._sensor_class = None
        self._sensor_icon = None
        self._sensor_unit = None
        super().__init__(device_id, device)

    def async_update(self, event, reason={}):
        """New event for sensor.

        Check if state or config is part of event.
        Signal that sensor has updated attributes.
        Inform what attributes got changed values.
        """
        reason['attr'] = []
        for data in ['state', 'config']:
            changed_attr = self.update_attr(event.get(data, {}))
            reason[data] = data in event
            reason['attr'] += changed_attr
        super().async_update(event, reason)

    @property
    def battery(self):
        """The battery status of the sensor."""
        return self._battery

    @property
    def ep(self):
        """The Endpoint of the sensor."""
        return self._ep

    @property
    def lowbattery(self):
        """Low battery."""
        return self._lowbattery

    @property
    def on(self):
        """Declare if the sensor is on or off."""
        return self._on

    @property
    def reachable(self):
        """Declare if the sensor is reachable."""
        return self._reachable

    @property
    def sensor_class(self):
        """Define what device class sensor belongs to."""
        return self._sensor_class

    @property
    def sensor_icon(self):
        """What Material Design icon should be used with this device."""
        return self._sensor_icon

    @property
    def sensor_unit(self):
        """What unit of measurement the sensor reports."""
        return self._sensor_unit

    @property
    def tampered(self):
        """Tampered."""
        return self._tampered

    # @property
    # def temperature(self):
    #     """Temperature."""
    #     return self._temperature


class Alarm(DeconzSensor):
    """Alarm sensor.

    {
        'config': {
            'battery': 100,
            'on': True,
            'reachable': True,
            'temperature': 2600
        },
        'ep': 1,
        'etag': '18c0f3c2100904e31a7f938db2ba9ba9',
        'manufacturername': 'dresden elektronik',
        'modelid': 'lumi.sensor_motion.aq2',
        'name': 'Alarm 10',
        'state': {
            'alarm': None,
            'lastupdated': 'none',
            'lowbattery': None,
            'tampered': None
        },
        'swversion': '20170627',
        'type': 'ZHAAlarm',
        'uniqueid': '00:15:8d:00:02:b5:d1:80-01-0500'
    }
    """

    def __init__(self, device_id, device):
        """Initialize alarm sensor."""
        self._alarm = device['state'].get('alarm')
        super().__init__(device_id, device)
        self._sensor_class = 'motion'

    @property
    def state(self):
        """Main state of sensor."""
        return self.alarm

    @property
    def alarm(self):
        """Alarm."""
        return self._alarm


class CarbonMonoxide(DeconzSensor):
    """Carbon monoxide sensor.

    State parameter is a boolean named 'carbonmonoxide'.
    {
        'config': {
            'battery': 100,
            'on': True,
            'pending': [],
            'reachable': True
        },
        'ep': 1,
        'etag': 'b7599df551944df97b2aa87d160b9c45',
        'manufacturername': 'Heiman',
        'modelid': 'CO_V16',
        'name': 'Cave, CO',
        'state': {
            'carbonmonoxide': False,
            'lastupdated': 'none',
            'lowbattery': False,
            'tampered': False
        },
        'swversion': '20150330',
        'type': 'ZHACarbonMonoxide',
        'uniqueid': '00:15:8d:00:02:a5:21:24-01-0101'
    }
    """

    def __init__(self, device_id, device):
        """Initialize Carbon monoxide sensor."""
        self._carbonmonoxide = device['state'].get('carbonmonoxide')
        super().__init__(device_id, device)
        self._sensor_class = 'carbon_monoxide'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def carbonmonoxide(self):
        """Fire detected."""
        return self._carbonmonoxide

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.carbonmonoxide


class Consumption(DeconzSensor):
    """Power consumption sensor.

    State parameter is a number named 'consumption'.
    {
        'config': {
            'on': True,
            'reachable': True
        },
        'ep': 1,
        'etag': 'a99e5bc463d15c23af7e89946e784cca',
        'manufacturername': 'Heiman',
        'modelid': 'SmartPlug',
        'name': 'Consumption 15',
        'state': {
            'consumption': 11342,
            'lastupdated': '2018-03-12T19:19:08'
        },
        'type': 'ZHAConsumption',
        'uniqueid': '00:0d:6f:00:0b:7a:64:29-01-0702'
    }
    """

    def __init__(self, device_id, device):
        """Initalize consumption sensor."""
        self._consumption = device['state'].get('consumption')
        super().__init__(device_id, device)
        self._sensor_unit = 'kWh'

    @property
    def state(self):
        """Main state of sensor."""
        if self.consumption is None:
            return None
        consumption = float(self.consumption/1000)
        return consumption

    @property
    def consumption(self):
        """Consumption."""
        return self._consumption


class Daylight(DeconzSensor):
    """Daylight sensor built into deCONZ software.

    State parameter is a string derived from 'status' parameter.
    Strings from daylight.h at
    https://github.com/dresden-elektronik/deconz-rest-plugin.
    Also has a 'daylight' boolean.
    Has no 'reachable' config parameter, so set sensor reachable True here.
    {
        "config": {
            "configured": true,
            "on": true,
            "sunriseoffset": 30,
            "sunsetoffset": -30
        },
        "etag": "55047cf652a7e594d0ee7e6fae01dd38",
        "manufacturername": "Philips",
        "modelid": "PHDL00",
        "name": "Daylight",
        "state": {
            "daylight": true,
            "lastupdated": "2018-03-24T17:26:12",
            "status": 170
        },
        "swversion": "1.0",
        "type": "Daylight"
    }
    """

    def __init__(self, device_id, device):
        """Initialize daylight sensor."""
        self._configured = device['state'].get('configured')
        self._daylight = device['state'].get('daylight')
        self._status = device['state'].get('status')
        self._sunriseoffset = device['config'].get('sunriseoffset')
        self._sunsetoffset = device['config'].get('sunsetoffset')
        super().__init__(device_id, device)
        self._reachable = True
        self._sensor_class = 'daylight'
        self._sensor_icon = 'mdi:white-balance-sunny'

    @property
    def state(self):
        """Main state of sensor."""
        return self.status

    @property
    def configured(self):
        """Is daylight sensor configured."""
        return self._configured

    @property
    def daylight(self):
        """True if daylight, false if not."""
        return self._daylight

    @property
    def status(self):
        """Return the daylight status string."""
        if self._status == 100:
            return "nadir"
        elif self._status == 110:
            return "night_end"
        elif self._status == 120:
            return "nautical_dawn"
        elif self._status == 130:
            return "dawn"
        elif self._status == 140:
            return "sunrise_start"
        elif self._status == 150:
            return "sunrise_end"
        elif self._status == 160:
            return "golden_hour_1"
        elif self._status == 170:
            return "solar_noon"
        elif self._status == 180:
            return "golden_hour_2"
        elif self._status == 190:
            return "sunset_start"
        elif self._status == 200:
            return "sunset_end"
        elif self._status == 210:
            return "dusk"
        elif self._status == 220:
            return "nautical_dusk"
        elif self._status == 230:
            return "night_start"
        else:
            return "unknown"

    @property
    def sunriseoffset(self):
        """Sunrise offset."""
        return self._sunriseoffset

    @property
    def sunsetoffset(self):
        """Sunset offset."""
        return self._sunsetoffset


class Fire(DeconzSensor):
    """Fire sensor.

    State parameter is a boolean named 'fire'.
    {
        "config": {
            "on": true,
            "reachable": true
        },
        "ep": 1,
        "etag": "2b585d2c016bfd665ba27a8fdad28670",
        "manufacturername": "LUMI",
        "modelid": "lumi.sensor_smoke",
        "name": "sensor_kitchen_smoke",
        "state": {
            "fire": false,
            "lastupdated": "2018-02-20T11:25:02"
        },
        "type": "ZHAFire",
        "uniqueid": "00:15:8d:00:01:d9:3e:7c-01-0500"
    }
    """

    def __init__(self, device_id, device):
        """Initialize Fire sensor."""
        self._fire = device['state'].get('fire')
        super().__init__(device_id, device)
        self._sensor_class = 'smoke'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def fire(self):
        """Fire detected."""
        return self._fire

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.fire


class GenericFlag(DeconzSensor):
    """Generic flag sensor.

    State parameter is a bool named 'flag'.
    {
        "config": {
            "on": true,
            "reachable": true
        },
        "modelid": "Switch",
        "name": "Kitchen Switch",
        "state": {
            "flag": true,
            "lastupdated": "2018-07-01T10:40:35"
        },
        "swversion": "1.0.0",
        "type": "CLIPGenericFlag",
        "uniqueid": "kitchen-switch"
    }
    """

    def __init__(self, device_id, device):
        """Initalize flag sensor."""
        self._flag = device['state'].get('flag')
        super().__init__(device_id, device)

    @property
    def state(self):
        """Main state of sensor."""
        return self.flag

    @property
    def flag(self):
        """Flag status."""
        return self._flag

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.flag


class GenericStatus(DeconzSensor):
    """Generic status sensor.

    State parameter is a number named 'status'.
    """

    def __init__(self, device_id, device):
        """Initalize generic sensor."""
        self._status = device['state'].get('status')
        super().__init__(device_id, device)

    @property
    def state(self):
        """Main state of sensor."""
        return self.status

    @property
    def status(self):
        """Status."""
        return self._status


class Humidity(DeconzSensor):
    """Humidity sensor.

    State parameter is a string named 'humidity'.
    """

    def __init__(self, device_id, device):
        """Initalize humidity sensor."""
        self._humidity = device['state'].get('humidity')
        super().__init__(device_id, device)
        self._sensor_class = 'humidity'
        self._sensor_unit = '%'

    @property
    def state(self):
        """Main state of sensor."""
        if self.humidity is None:
            return None
        humidity = round(float(self.humidity) / 100, 1)
        return humidity

    @property
    def humidity(self):
        """Humidity level."""
        return self._humidity


class LightLevel(DeconzSensor):
    """Light level sensor.

    State parameter is a string named lightlevel.
    {
        'config': {
            'battery': 100,
            'on': True,
            'reachable': True,
            'temperature': 2600,
            'tholddark': 12000,
            'tholdoffset': 7000
        },
        'ep': 1,
        'etag': '26e97d94b471c1799a1a5951cee7938b',
        'manufacturername': 'dresden elektronik',
        'modelid': 'lumi.sensor_motion.aq2',
        'name': 'LightLevel 9',
        'state': {
            'dark': True,
            'daylight': False,
            'lastupdated': '2019-01-29T07:19:53',
            'lightlevel': 0,
            'lux': 0
        },
        'swversion': '20170627',
        'type': 'ZHALightLevel',
        'uniqueid': '00:15:8d:00:02:b5:d1:80-01-0400'
    }
    """

    def __init__(self, device_id, device):
        """Initalize light level sensor."""
        self._dark = device['state'].get('dark')
        self._daylight = device['state'].get('daylight')
        self._lightlevel = device['state'].get('lightlevel')
        self._lux = device['state'].get('lux')
        self._tholddark = device['config'].get('tholddark')
        self._tholdoffset = device['config'].get('tholdoffset')
        super().__init__(device_id, device)
        self._sensor_class = 'lux'
        self._sensor_unit = 'lux'

    @property
    def state(self):
        """Main state of sensor."""
        if self.lightlevel is None:
            return None
        lux = round(10 ** (float(self.lightlevel - 1) / 10000), 1)
        return lux

    @property
    def dark(self):
        """If the area near the sensor is light or not."""
        return self._dark

    @property
    def daylight(self):
        """Daylight."""
        return self._daylight

    @property
    def lightlevel(self):
        """Light level."""
        return self._lightlevel

    @property
    def lux(self):
        """Lux."""
        return self._lux

    @property
    def tholddark(self):
        """Threshold to hold dark."""
        return self._tholddark

    @property
    def tholdoffset(self):
        """Offset for threshold to hold dark."""
        return self._tholdoffset


class OpenClose(DeconzSensor):
    """Door/Window sensor.

    State parameter is a boolean named 'open'.
    """

    def __init__(self, device_id, device):
        """Initialize Door/Window sensor."""
        self._open = device['state'].get('open')
        super().__init__(device_id, device)
        self._sensor_class = 'opening'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.open

    @property
    def open(self):
        """Door open."""
        return self._open


class Power(DeconzSensor):
    """Power sensor.

    State parameter is a number named 'power'.
    {
        'config': {
            'on': True,
            'reachable': True
        },
        'ep': 1,
        'etag': '96e71c7db4685b334d3d0decc3f11868',
        'manufacturername': 'Heiman',
        'modelid': 'SmartPlug',
        'name': 'Power 16',
        'state': {
            'current': 34,
            'lastupdated': '2018-03-12T19:22:13',
            'power': 64,
            'voltage': 231
        },
        'type': 'ZHAPower',
        'uniqueid': '00:0d:6f:00:0b:7a:64:29-01-0b04'
    }
    """

    def __init__(self, device_id, device):
        """Initalize power sensor."""
        self._current = device['state'].get('current')
        self._power = device['state'].get('power')
        self._voltage = device['state'].get('voltage')
        super().__init__(device_id, device)
        self._sensor_unit = 'Watts'

    @property
    def state(self):
        """Main state of sensor."""
        return self.power

    @property
    def current(self):
        """Current."""
        return self._current

    @property
    def power(self):
        """Power."""
        return self._power

    @property
    def voltage(self):
        """Voltage."""
        return self._voltage


class Presence(DeconzSensor):
    """Presence detector.

    State parameter is a boolean named 'presence'.
    Also has a boolean 'dark' representing lighting in area of placement.
    {
        'config': {
            'battery': 100,
            'duration': 60,
            'on': True,
            'reachable': True,
            'temperature': 2600
        },
        'ep': 1,
        'etag': '26e97d94b471c1799a1a5951cee7938b',
        'manufacturername': 'dresden elektronik',
        'modelid': 'lumi.sensor_motion.aq2',
        'name': 'presence1',
        'state': {
            'lastupdated': '2019-01-29T07:19:53',
            'presence': False
        },
        'swversion': '20170627',
        'type': 'ZHAPresence',
        'uniqueid': '00:15:8d:00:02:b5:d1:80-01-0406'}
    """

    def __init__(self, device_id, device):
        """Initialize presence detector."""
        self._dark = device['state'].get('dark')
        self._duration = device['config'].get('duration')
        self._presence = device['state'].get('presence')
        super().__init__(device_id, device)
        self._sensor_class = 'motion'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def dark(self):
        """If the area near the sensor is light or not."""
        return self._dark

    @property
    def duration(self):
        """Minimum duration which presence will be true."""
        return self._duration

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.presence

    @property
    def presence(self):
        """Motion detected."""
        return self._presence


class Pressure(DeconzSensor):
    """Pressure sensor.

    State parameter is a string named 'pressure'.
    """

    def __init__(self, device_id, device):
        """Initalize pressure sensor."""
        self._pressure = device['state'].get('pressure')
        super().__init__(device_id, device)
        self._sensor_class = 'pressure'
        self._sensor_icon = 'mdi:gauge'
        self._sensor_unit = 'hPa'

    @property
    def state(self):
        """Main state of sensor."""
        return self.pressure

    @property
    def pressure(self):
        """Pressure."""
        return self._pressure


class Switch(DeconzSensor):
    """Switch.

    State parameter is a string named 'buttonevent'.
    """

    def __init__(self, device_id, device):
        """Initalize switch sensor."""
        self._buttonevent = device['state'].get('buttonevent')
        super().__init__(device_id, device)

    @property
    def state(self):
        """Main state of switch."""
        return self.buttonevent

    @property
    def buttonevent(self):
        """Button press."""
        return self._buttonevent


class Temperature(DeconzSensor):
    """Temperature sensor.

    State parameter is a string named 'temperature'.
    """

    def __init__(self, device_id, device):
        """Initalize temperature sensor."""
        self._temperature = device['state'].get('temperature')
        super().__init__(device_id, device)
        self._sensor_class = 'temperature'
        self._sensor_icon = 'mdi:thermometer'
        self._sensor_unit = '°C'

    @property
    def state(self):
        """Main state of sensor."""
        return self.temperature

    @property
    def temperature(self):
        """Temperature."""
        return self.convert_temperature(self._temperature)

    def convert_temperature(self, temperature):
        """Convert temperature to celsius"""
        if temperature is None:
            return None
        return round(float(temperature) / 100, 1)


class Thermostat(Temperature, DeconzDeviceSetter):
    """Thermostat "sensor".

    State parameter is a string named 'temperature'.
    {
        "config": {
            "battery": 100,
            "displayflipped": true,
            "heatsetpoint": 2100,
            "locked": false,
            "mode": "auto",
            "offset": 0,
            "on": true,
            "reachable": true
        },
        "ep": 1,
        "etag": "25aac331bc3c4b465cfb2197f6243ea4",
        "manufacturername": "Eurotronic",
        "modelid": "SPZB0001",
        "name": "Living Room Radiator",
        "state": {
            "lastupdated": "2019-02-10T22:41:32",
            "on": false,
            "temperature": 2149,
            "valve": 0
        },
        "swversion": "15181120",
        "type": "ZHAThermostat",
        "uniqueid": "00:15:8d:00:01:92:d2:51-01-0201"
    }
    """

    def __init__(self, device_id, device, loop, async_set_state_callback):
        """Initalize temperature sensor."""
        self._heatsetpoint = device['config'].get('heatsetpoint')
        self._locked = device['config'].get('locked')
        self._mode = device['config'].get('mode')
        self._offset = device['config'].get('offset')
        self._valve = device['state'].get('valve')

        Temperature.__init__(self, device_id, device)
        DeconzDeviceSetter.__init__(self, async_set_state_callback, loop)

        self._on = device['state'].get('on')

    async def async_set_config(self, data):
        """Set config of thermostat.

        {
            "mode": "auto",
            "heatsetpoint": 180,
        }
        """
        field = self.deconz_id + '/config'

        await self.async_set(field, data)

    @property
    def heatsetpoint(self):
        """Heating setpoint"""
        return self.convert_temperature(self._heatsetpoint)

    @property
    def mode(self):
        """Thermostat mode; "off", "heat", and "auto"."""
        return self._mode

    @property
    def offset(self):
        """Temperature offset."""
        return self._offset

    @property
    def valve(self):
        """How open the valve is."""
        return self._valve


class Vibration(DeconzSensor):
    """Vibration sensor.

    State parameter is a string named 'vibration'.
    {
        "config": {
            "battery": 91,
            "on": true,
            "pending": [],
            "reachable": true,
            "sensitivity": 21,
            "sensitivitymax": 21,
            "temperature": 3200
        },
        "ep": 1,
        "etag": "b7599df551944df97b2aa87d160b9c45",
        "manufacturername": "LUMI",
        "modelid": "lumi.vibration.aq1",
        "name": "Vibration 9",
        "state": {
            "lastupdated": "2019-03-09T15:53:07",
            "orientation": [
                10,
                1059,
                0
            ],
            "tiltangle": 83,
            "vibration": true,
            "vibrationstrength": 114
        },
        "swversion": "20180130",
        "type": "ZHAVibration",
        "uniqueid": "00:15:8d:00:02:a5:21:24-01-0101"
    }
    """

    def __init__(self, device_id, device):
        """Initalize vibration sensor."""
        self._sensitivity = device['config'].get('sensitivity')
        self._sensitivitymax = device['config'].get('sensitivitymax')
        self._orientation = device['state'].get('orientation')
        self._tiltangle = device['state'].get('tiltangle')
        self._vibration = device['state'].get('vibration')
        self._vibrationstrength = device['state'].get('vibrationstrength')
        super().__init__(device_id, device)
        self._sensor_class = 'motion'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def is_tripped(self) -> bool:
        """Sensor is tripped."""
        return self.vibration

    @property
    def orientation(self) -> list:
        """Orientation."""
        return self._orientation

    @property
    def sensitivity(self) -> int:
        """Configured sensitivity"""
        return self._sensitivity

    @property
    def sensitivitymax(self) -> int:
        """Configured max sensitivity."""
        return self._sensitivitymax

    @property
    def tiltangle(self) -> int:
        """Tilt angle."""
        return self._tiltangle

    @property
    def vibration(self) -> bool:
        """Vibration."""
        return self._vibration

    @property
    def vibrationstrength(self) -> int:
        """Strength of vibration."""
        return self._vibrationstrength


class Water(DeconzSensor):
    """Water sensor.

    State parameter is a boolean named 'water'.
    {
        'config': {
            'battery': 100,
            'on': True,
            'reachable': True,
            'temperature': 2500
        },
        'ep': 1,
        'etag': 'fae893708dfe9b358df59107d944fa1c',
        'manufacturername': 'LUMI',
        'modelid': 'lumi.sensor_wleak.aq1',
        'name': 'water2',
        'state': {
            'lastupdated': '2019-01-29T07:13:20',
            'lowbattery': False,
            'tampered': False,
            'water': False
        },
        'swversion': '20170721',
        'type': 'ZHAWater',
        'uniqueid': '00:15:8d:00:02:2f:07:db-01-0500'
    }
    """

    def __init__(self, device_id, device):
        """Initialize Water sensor."""
        self._water = device['state'].get('water')
        super().__init__(device_id, device)
        self._sensor_class = 'moisture'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.water

    @property
    def water(self):
        """Water detected."""
        return self._water


def create_sensor(
        sensor_id, sensor, loop, async_set_state_callback):
    """Simplify creating sensor by not needing to know type."""
    if sensor['type'] in CONSUMPTION:
        return Consumption(sensor_id, sensor)
    if sensor['type'] in CARBONMONOXIDE:
        return CarbonMonoxide(sensor_id, sensor)
    if sensor['type'] in DAYLIGHT:
        return Daylight(sensor_id, sensor)
    if sensor['type'] in FIRE:
        return Fire(sensor_id, sensor)
    if sensor['type'] in GENERICFLAG:
        return GenericFlag(sensor_id, sensor)
    if sensor['type'] in GENERICSTATUS:
        return GenericStatus(sensor_id, sensor)
    if sensor['type'] in HUMIDITY:
        return Humidity(sensor_id, sensor)
    if sensor['type'] in LIGHTLEVEL:
        return LightLevel(sensor_id, sensor)
    if sensor['type'] in OPENCLOSE:
        return OpenClose(sensor_id, sensor)
    if sensor['type'] in POWER:
        return Power(sensor_id, sensor)
    if sensor['type'] in PRESENCE:
        return Presence(sensor_id, sensor)
    if sensor['type'] in PRESSURE:
        return Pressure(sensor_id, sensor)
    if sensor['type'] in SWITCH:
        return Switch(sensor_id, sensor)
    if sensor['type'] in TEMPERATURE:
        return Temperature(sensor_id, sensor)
    if sensor['type'] in THERMOSTAT:
        return Thermostat(sensor_id, sensor, loop, async_set_state_callback)
    if sensor['type'] in VIBRATION:
        return Vibration(sensor_id, sensor)
    if sensor['type'] in WATER:
        return Water(sensor_id, sensor)


def supported_sensor(sensor):
    """Check if sensor is supported by pydeconz."""
    if sensor['type'] in DECONZ_BINARY_SENSOR + DECONZ_SENSOR + OTHER_SENSOR:
        return True
    _LOGGER.info('Unsupported sensor type %s (%s)',
                 sensor['type'], sensor['name'])
    return False



""""""

import aiohttp
import asyncio

from pydeconz import DeconzSession
from pydeconz.utils import (async_get_api_key, async_delete_all_keys)


async def main(loop, **kwargs):
    """
    """
    if 'api_key' not in kwargs:
        api_key = await async_get_api_key(loop, **kwargs)
        kwargs['api_key'] = api_key
        print(api_key)
    websession = aiohttp.ClientSession(loop=loop)
    deconz = DeconzSession(loop, websession, **kwargs)
    result = await deconz.async_load_parameters()
    if result is False:
        print('Failed to setup deCONZ')
        return False
    deconz.start()
    from pprint import pprint
    pprint(deconz.__dict__)
    #await deconz.async_delete_state('/lights/2/groups', {'reset':'true'})
    #for dev_id, dev in deconz.config.values():
    #    pprint(dev.__dict__)
    # await deconz.close()
    # await async_delete_api_key(loop, **kwargs)
    # await async_delete_all_keys(loop, **kwargs)


kw = {'host': '10.0.0.10',
      'port': 8088,
      'api_key': '8BA2DD354B',
      #'username': 'delight',
      #'password': 'delight'
      }

loop = asyncio.get_event_loop()
loop.run_until_complete(main(loop, **kw))
loop.run_forever()
loop.close()



"""Python library to connect deCONZ and Home Assistant to work together."""

import asyncio
import logging
import aiohttp

from .errors import raise_error, ResponseError, RequestError

_LOGGER = logging.getLogger(__name__)

URL_DISCOVER = 'https://dresden-light.appspot.com/discover'


async def async_get_api_key(session, host, port, username=None, password=None, **kwargs):
    """Get a new API key for devicetype."""
    url = 'http://{host}:{port}/api'.format(host=host, port=str(port))

    auth = None
    if username and password:
        auth = aiohttp.BasicAuth(username, password=password)

    data = b'{"devicetype": "pydeconz"}'
    response = await async_request(session.post, url, auth=auth, data=data)

    api_key = response[0]['success']['username']
    _LOGGER.info("API key: %s", api_key)
    return api_key


async def async_delete_api_key(session, host, port, api_key):
    """Delete API key from deCONZ."""
    url = 'http://{host}:{port}/api/{api_key}/config/whitelist/{api_key}'.format(
        host=host, port=str(port), api_key=api_key)

    response = await async_request(session.delete, url)

    _LOGGER.info(response)


async def async_delete_all_keys(session, host, port, api_key, api_keys=[]):
    """Delete all API keys except for the ones provided to the method."""
    url = 'http://{}:{}/api/{}/config'.format(host, str(port), api_key)

    response = await async_request(session.get, url)

    api_keys.append(api_key)
    for key in response['whitelist'].keys():
        if key not in api_keys:
            await async_delete_api_key(session, host, port, key)


async def async_get_bridgeid(session, host, port, api_key, **kwargs):
    """Get bridge id for bridge."""
    url = 'http://{}:{}/api/{}/config'.format(host, str(port), api_key)

    response = await async_request(session.get, url)

    bridgeid = response['bridgeid']
    _LOGGER.info("Bridge id: %s", bridgeid)
    return bridgeid


async def async_discovery(session):
    """Find bridges allowing gateway discovery."""
    bridges = []
    response = await async_request(session.get, URL_DISCOVER)

    if not response:
        _LOGGER.info("No discoverable bridges available.")
        return bridges

    for bridge in response:
        bridges.append({'bridgeid': bridge['id'],
                        'host': bridge['internalipaddress'],
                        'port': bridge['internalport']})

    _LOGGER.info("Discovered the following bridges: %s.", bridges)

    return bridges


async def async_request(session, url, **kwargs):
    """Do a web request and manage response."""
    _LOGGER.debug("Sending %s to %s", kwargs, url)

    try:
        res = await session(url, **kwargs)

        if res.content_type != 'application/json':
            raise ResponseError(
                "Invalid content type: {}".format(res.content_type))

        response = await res.json()
        _LOGGER.debug("HTTP request response: %s", response)

        _raise_on_error(response)

        return response

    except aiohttp.client_exceptions.ClientError as err:
        raise RequestError(
            "Error requesting data from {}: {}".format(url, err)
        ) from None


def _raise_on_error(data):
    """Check response for error message."""
    if isinstance(data, list) and data:
        data = data[0]

    if isinstance(data, dict) and 'error' in data:
        raise_error(data['error'])



"""Python library to connect deCONZ and Home Assistant to work together."""

import json
import logging
from pprint import pformat

from .config import DeconzConfig
from .group import DeconzGroup
from .light import DeconzLight
from .sensor import create_sensor, supported_sensor
from .utils import async_request

_LOGGER = logging.getLogger(__name__)

class DeconzSession:
    """deCONZ representation that handles lights, groups, scenes and sensors."""

    def __init__(self, loop, websession, host, port, api_key, **kwargs):
        """Setup session and host information."""
        self.groups = {}
        self.lights = {}
        self.scenes = {}
        self.sensors = {}
        self.config = None
        self.loop = loop
        self.session = websession
        self.host = host
        self.api_url = 'http://{}:{}/api/{}'.format(host, port, api_key)
        if 'legacy_websocket' in kwargs:
            from .websocket import WSClient as ws_client
        else:
            from .websocket import AIOWSClient as ws_client
        self.ws_client = ws_client
        self.websocket = None
        self.async_add_device_callback = kwargs.get('async_add_device')
        self.async_connection_status_callback = kwargs.get('connection_status')

    def start(self) -> None:
        """Connect websocket to deCONZ."""
        if self.config:
            self.websocket = self.ws_client(
                self.loop, self.session, self.host,
                self.config.websocketport, self.async_session_handler)
            self.websocket.start()
        else:
            _LOGGER.error('No deCONZ config available')

    def close(self) -> None:
        """Close websession and websocket to deCONZ."""
        _LOGGER.info('Shutting down connections to deCONZ.')
        if self.websocket:
            self.websocket.stop()

    async def async_load_parameters(self) -> bool:
        """Load deCONZ parameters.

        Returns lists of indices of which devices was added.
        """
        data = await self.async_get_state('')

        _LOGGER.debug(pformat(data))

        config = data.get('config', {})
        groups = data.get('groups', {})
        lights = data.get('lights', {})
        sensors = data.get('sensors', {})

        if not self.config:
            self.config = DeconzConfig(config)

        # Update scene for existing groups
        for group_id, group in groups.items():
            if group_id in self.groups:
                self.groups[group_id].async_add_scenes(
                    group.get('scenes'), self.async_put_state)

        self.groups.update({
            group_id: DeconzGroup(group_id, group,
                                  self.loop, self.async_put_state)
            for group_id, group in groups.items()
            if group_id not in self.groups
        })

        self.lights.update({
            light_id: DeconzLight(light_id, light,
                                  self.loop, self.async_put_state)
            for light_id, light in lights.items()
            if light_id not in self.lights
        })
        self.update_group_color(self.lights.keys())

        self.scenes.update({
            group.id + '_' + scene.id: scene
            for group in self.groups.values()
            for scene in group.scenes.values()
            if group.id + '_' + scene.id not in self.scenes
        })

        self.sensors.update({
            sensor_id: create_sensor(sensor_id, sensor,
                                     self.loop, self.async_put_state)
            for sensor_id, sensor in sensors.items()
            if supported_sensor(sensor) and sensor_id not in self.sensors
        })

    async def async_put_state(self, field: str, data: dict) -> dict:
        """Set state of object in deCONZ.

        Field is a string representing a specific device in deCONZ
        e.g. field='/lights/1/state'.
        Data is a json object with what data you want to alter
        e.g. data={'on': True}.
        See Dresden Elektroniks REST API documentation for details:
        http://dresden-elektronik.github.io/deconz-rest-doc/rest/
        """
        session = self.session.put
        url = self.api_url + field
        jsondata = json.dumps(data)
        response_dict = await async_request(session, url, data=jsondata)
        return response_dict

    async def async_get_state(self, field: str) -> dict:
        """Get state of object in deCONZ.

        Field is a string representing an API endpoint or lower
        e.g. field='/lights'.
        See Dresden Elektroniks REST API documentation for details:
        http://dresden-elektronik.github.io/deconz-rest-doc/rest/
        """
        session = self.session.get
        url = self.api_url + field
        response_dict = await async_request(session, url)
        return response_dict

    def async_session_handler(self, signal: str) -> None:
        """Signalling from websocket.

           data - new data available for processing.
           state - network state has changed.
        """
        if signal == 'data':
            self.async_event_handler(self.websocket.data)
        elif signal == 'state':
            if self.async_connection_status_callback:
                self.async_connection_status_callback(
                    self.websocket.state == 'running')

    def async_event_handler(self, event: dict) -> None:
        """Receive event from websocket and identifies where the event belong.

        {
            "t": "event",
            "e": "changed",
            "r": "sensors",
            "id": "12",
            "state": { "buttonevent": 2002 }
        }
        """
        if event['e'] == 'added':

            if event['r'] == 'lights' and event['id'] not in self.lights:
                device_type = 'light'
                device = self.lights[event['id']] = DeconzLight(
                    event['id'], event['light'],
                    self.loop, self.async_put_state)

            elif event['r'] == 'sensors' and event['id'] not in self.sensors:
                if supported_sensor(event['sensor']):
                    device_type = 'sensor'
                    device = self.sensors[event['id']] = create_sensor(
                        event['id'], event['sensor'],
                        self.loop, self.async_put_state)
                else:
                    _LOGGER.warning('Unsupported sensor %s', event)
                    return

            else:
                _LOGGER.debug('Unsupported event %s', event)
                return

            if self.async_add_device_callback:
                self.async_add_device_callback(device_type, device)

        elif event['e'] == 'changed':

            if event['r'] == 'groups' and event['id'] in self.groups:
                self.groups[event['id']].async_update(event)

            elif event['r'] == 'lights' and event['id'] in self.lights:
                self.lights[event['id']].async_update(event)
                self.update_group_color([event['id']])

            elif event['r'] == 'sensors' and event['id'] in self.sensors:
                self.sensors[event['id']].async_update(event)

            else:
                _LOGGER.debug('Unsupported event %s', event)

        elif event['e'] == 'deleted':
            _LOGGER.debug('Removed event %s', event)

        else:
            _LOGGER.debug('Unsupported event %s', event)

    def update_group_color(self, lights: list) -> None:
        """Update group colors based on light states.

        deCONZ group updates don't contain any information about the current
        state of the lights in the group. This method updates the color
        properties of the group to the current color of the lights in the
        group.

        For groups where the lights have different colors the group color will
        only reflect the color of the latest changed light in the group.
        """
        for group in self.groups.values():
            # Skip group if there are no common light ids.
            if not any({*lights} & {*group.lights}):
                continue

            # More than one light means load_parameters called this method.
            # Then we take first best light to be available.
            light_ids = lights
            if len(light_ids) > 1:
                light_ids = group.lights

            for light_id in light_ids:
                if self.lights[light_id].reachable:
                    group.update_color_state(self.lights[light_id])
                    break



"""Setup for deCONZ"""

# https://jeffknupp.com/blog/2013/08/16/open-sourcing-a-python-project-the-right-way/
# http://peterdowns.com/posts/first-time-with-pypi.html
# pip install -e .
# Upload to PyPI Live
# python setup.py sdist bdist_wheel
# twine upload dist/pydeconz-* --skip-existing

from setuptools import setup

setup(
    name='pydeconz',
    packages=['pydeconz'],
    version='57',
    description='A Python library for communicating with deCONZ REST-API from Dresden Elektronik',
    author='Robert Svensson',
    author_email='Kane610@users.noreply.github.com',
    license='MIT',
    url='https://github.com/Kane610/deconz',
    download_url='https://github.com/Kane610/deconz/archive/v57.tar.gz',
    install_requires=['aiohttp'],
    keywords=['deconz', 'zigbee', 'homeassistant'],
    classifiers=[
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
    ],
)



"""Python library to connect deCONZ and Home Assistant to work together."""

import asyncio
import json
import logging
import os

import aiohttp

from base64 import encodestring as base64encode
from struct import unpack

_LOGGER = logging.getLogger(__name__)

STATE_STARTING = 'starting'
STATE_RUNNING = 'running'
STATE_STOPPED = 'stopped'

RETRY_TIMER = 15


class AIOWSClient:
    """Websocket transport, session handling, message generation."""

    def __init__(self, loop, session, host, port, async_callback):
        """Create resources for websocket communication."""
        self.loop = loop
        self.session = session
        self.host = host
        self.port = port
        self.async_session_handler_callback = async_callback
        self._data = None
        self._state = None

    @property
    def data(self):
        return self._data

    @property
    def state(self):
        """"""
        return self._state

    @state.setter
    def state(self, value):
        """"""
        self._state = value
        _LOGGER.debug('Websocket %s', value)
        self.async_session_handler_callback('state')

    def start(self):
        if self.state != STATE_RUNNING:
            self.state = STATE_STARTING
            self.loop.create_task(self.running())

    async def running(self):
        """Start websocket connection."""
        url = 'http://{}:{}'.format(self.host, self.port)
        try:
            async with self.session.ws_connect(url) as ws:
                self.state = STATE_RUNNING
                async for msg in ws:
                    if self.state == STATE_STOPPED:
                        break
                    elif msg.type == aiohttp.WSMsgType.TEXT:
                        self._data = json.loads(msg.data)
                        self.async_session_handler_callback('data')
                        _LOGGER.debug('Websocket data: %s', msg.data)
                    elif msg.type == aiohttp.WSMsgType.CLOSED:
                        break
                    elif msg.type == aiohttp.WSMsgType.ERROR:
                        break
        except aiohttp.ClientConnectorError:
            if self.state != STATE_STOPPED:
                self.retry()
        except Exception as err:
            _LOGGER.error('Unexpected error %s', err)
            if self.state != STATE_STOPPED:
                self.retry()
        else:
            if self.state != STATE_STOPPED:
                self.retry()

    def stop(self):
        """Close websocket connection."""
        self.state = STATE_STOPPED

    def retry(self):
        """Retry to connect to deCONZ."""
        self.state = STATE_STARTING
        self.loop.call_later(RETRY_TIMER, self.start)
        _LOGGER.debug('Reconnecting to deCONZ in %i.', RETRY_TIMER)


class WSClient(asyncio.Protocol):
    """Websocket transport, session handling, message generation."""

    def __init__(self, loop, session, host, port, async_callback):
        """Create resources for websocket communication."""
        self.loop = loop
        self.host = host
        self.port = port
        self.async_session_handler_callback = async_callback
        self._data = None
        self._state = None
        self.transport = None
        _LOGGER.warning('Using legacy websocket, this is not recommended')

    def start(self):
        """Start websocket connection."""
        if self.state != STATE_RUNNING:
            conn = self.loop.create_connection(
                lambda: self, self.host, self.port)
            task = self.loop.create_task(conn)
            task.add_done_callback(self.init_done)
            self.state = STATE_STARTING

    def init_done(self, fut):
        """Server ready.

        If we get OSError during init the device is not available.
        """
        try:
            if fut.exception():
                fut.result()
        except OSError as err:
            _LOGGER.debug('Got exception %s', err)
            self.retry()

    @property
    def data(self):
        return self._data

    @property
    def state(self):
        """"""
        return self._state

    @state.setter
    def state(self, value):
        """"""
        self._state = value
        _LOGGER.debug('Websocket %s', value)
        self.async_session_handler_callback('state')

    def stop(self):
        """Close websocket connection."""
        self.state = STATE_STOPPED
        if self.transport:
            self.transport.close()

    def retry(self):
        """Retry to connect to deCONZ."""
        self.loop.call_later(RETRY_TIMER, self.start)
        _LOGGER.debug('Reconnecting to deCONZ in %i.', RETRY_TIMER)

    def connection_made(self, transport):
        """Do the websocket handshake.

        According to https://tools.ietf.org/html/rfc6455
        """
        randomness = os.urandom(16)
        key = base64encode(randomness).decode('utf-8').strip()
        self.transport = transport
        message = "GET / HTTP/1.1\r\n"
        message += "Host: " + self.host + ':' + str(self.port) + '\r\n'
        message += "User-Agent: Python/3.5 websockets/3.4\r\n"
        message += "Upgrade: Websocket\r\n"
        message += "Connection: Upgrade\r\n"
        message += "Sec-WebSocket-Key: " + key + "\r\n"
        message += "Sec-WebSocket-Version: 13\r\n"
        message += "\r\n"
        _LOGGER.debug('Websocket handshake: %s', message)
        self.transport.write(message.encode())

    def data_received(self, data):
        """Data received over websocket.

        First received data will allways be handshake accepting connection.
        We need to check how big the header is so we can send event data
        as a proper json object.
        """
        if self.state == STATE_STARTING:
            self.state = STATE_RUNNING
            _LOGGER.debug('Websocket handshake: %s', data.decode())
            return
        _LOGGER.debug('Websocket data: %s', data)

        while len(data) > 0:
            payload, extra_data = self.get_payload(data)
            self._data = payload ###
            self.async_session_handler_callback('data')###
            #self.async_callback(payload)
            data = extra_data

    def connection_lost(self, exc):
        """Happen when device closes connection or stop() has been called."""
        if self.state == STATE_RUNNING:
            _LOGGER.warning('Lost connection to deCONZ')
            self.retry()

    def get_payload(self, data):
        """Parse length of payload and return it."""
        start = 2
        length = ord(data[1:2])
        if length == 126:
            # Payload information are an extra 2 bytes.
            start = 4
            length, = unpack(">H", data[2:4])
        elif length == 127:
            # Payload information are an extra 6 bytes.
            start = 8
            length, = unpack(">I", data[2:6])
        end = start + length
        payload = json.loads(data[start:end].decode())
        extra_data = data[end:]
        return payload, extra_data



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

_LOGGER = logging.getLogger(__name__)


class DeconzConfig:
    """deCONZ configuration representation.

    Dresden Elektroniks documentation of config in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/config/
    {
        'UTC': '2017-11-04T12:01:19',
        'apiversion': '1.0.4',
        'backup': {'errorcode': 0, 'status': 'idle'},
        'bridgeid': '0123456789ABCDEF',
        'datastoreversion': '60',
        'dhcp': True,
        'factorynew': False,
        'gateway': '192.168.0.1',
        'internetservices': {'remoteaccess': 'disconnected'},
        'ipaddress': '192.168.0.90',
        'linkbutton': False,
        'localtime': '2017-11-04T13:01:19',
        'mac': '00:11:22:33:44:55',
        'modelid': 'deCONZ',
        'name': 'deCONZ-GW',
        'netmask': '255.255.255.0',
        'networkopenduration': 60,
        'panid': 50436,
        'portalconnection': 'disconnected',
        'portalservices': False,
        'portalstate': {'communication': 'disconnected',
                        'incoming': False,
                        'outgoing': False,
                        'signedon': False},
        'proxyaddress': 'none',
        'proxyport': 0,
        'replacesbridgeid': None,
        'starterkitid': '',
        'swupdate': {'checkforupdate': False,
                     'devicetypes': {'bridge': False, 'lights': [], 'sensors': []},
                     'notify': False,
                     'text': '',
                     'updatestate': 0,
                     'url': ''},
        'swversion': '2.4.82',
        'timeformat': '24h',
        'timezone': 'Europe/Stockholm',
        'uuid': '12345678-90AB-CDEF-1234-1234567890AB',
        'websocketnotifyall': True,
        'websocketport': 443,
        'whitelist': {'1234567890': {'create date': '2017-11-02T23:13:13',
                                     'last use date': '2017-11-04T12:00:03',
                                     'name': 'deCONZ WebApp'},},
        'wifi': 'not-installed',
        'wifiappw': '',
        'wifichannel': '1',
        'wifiip': '192.168.8.1',
        'wifiname': 'Not set',
        'wifitype': 'accesspoint',
        'zigbeechannel': 11
    }
    """

    def __init__(self, config):
        """Set initial information about light.

        Set callback to set state of device.
        """
        self._apiversion = config.get('apiversion')
        self._bridgeid = config.get('bridgeid')
        self._linkbutton = config.get('linkbutton')
        self._mac = config.get('mac')
        self._modelid = config.get('modelid')
        self._name = config.get('name')
        self._networkopenduration = config.get('networkopenduration')
        self._panid = config.get('panid')
        self._swversion = config.get('swversion')
        self._uuid = config.get('uuid')
        self._websocketport = config.get('websocketport')
        self._zigbeechannel = config.get('zigbeechannel')
        _LOGGER.debug('Deconz config loaded %s', self.__dict__)

    @property
    def apiversion(self):
        """The version of the deCONZ Rest API."""
        return self._apiversion

    @property
    def bridgeid(self):
        """Hardware ID."""
        return self._bridgeid

    @property
    def linkbutton(self):
        """True if the gateway is unlocked."""
        return self._linkbutton

    @property
    def mac(self):
        """MAC address of the gateway"""
        return self._mac

    @property
    def modelid(self):
        return self._modelid

    @property
    def name(self):
        """Name of the gateway."""
        return self._name

    @property
    def networkopenduration(self):
        """Can be used to store the permitjoin value permanently."""
        return self._networkopenduration

    @property
    def panid(self):
        """The ZigBee pan ID of the gateway."""
        return self._panid

    @property
    def swversion(self):
        """The software version of the gateway."""
        return self._swversion

    @property
    def uuid(self):
        """UPNP Unique Id of the gateway."""
        return self._uuid

    @property
    def websocketport(self):
        """Websocket port."""
        return self._websocketport

    @property
    def zigbeechannel(self):
        """The current wireless frequency channel used by the Gateway.

        Supported channels: 11, 15, 20, 25."""
        return self._zigbeechannel



"""deCONZ errors."""


class pydeconzException(Exception):
    """Base error for pydeconz.

    https://dresden-elektronik.github.io/deconz-rest-doc/errors/
    """


class RequestError(pydeconzException):
    """Unable to fulfill request.

    Raised when host or API cannot be reached.
    """


class ResponseError(pydeconzException):
    """Invalid response."""


class Unauthorized(pydeconzException):
    """Authorization failed."""


class BadRequest(pydeconzException):
    """The request was not formatted as expected or missing parameters."""


class Forbidden(pydeconzException):
    """The caller has no rights to access the requested URI."""


class ResourceNotFound(pydeconzException):
    """The requested resource (light, group, ...) was not found."""


class BridgeBusy(pydeconzException):
    """The Bridge is busy, too many requests (more than 20)."""


ERRORS = {
    1: Unauthorized,  # Unauthorized user
    2: BadRequest,  # Body contains invalid JSON
    3: ResourceNotFound,  # Resource not available
    4: RequestError,  # Method not available for resource
    5: BadRequest,  # Missing parameters in body
    6: RequestError,  # Parameter not available
    7: RequestError,  # Invalid value for parameter
    8: RequestError,  # Parameter is not modifiable
    901: BridgeBusy,  # May occur when sending too fast
}


def raise_error(error):
    if error:
        cls = ERRORS.get(error['type'], pydeconzException)
        raise cls("{} {}".format(error['address'], error['description']))
